/*************************************************************************/
/*************************************************************************/
/* H. Timucin OZDEMIR                                                    */
/* Date : February, 1997                                                 */
/* File Name : GeneticAlgorithm.h                                        */
/*************************************************************************/
/*************************************************************************/

#ifndef _GeneticAlgoritm
#define _GeneticAlgoritm

enum OP_TYPE { MUTATION, CROSSOVER };

#include <stdio.h>
#include "Population.h"
#include "MutationOperator.h"
#include "CrossOverOperation.h"
#include "FitnessFunction.h"
#include "ParentSelector.h"
#include "Recombinator.h"
#include "Initializer.h"
#include "RandomGenerator.h"
#include "GAOperationsBag.h"
#include "Sampler.h"


class GeneticAlgorithm
{  
   public :

       void run();
       void runFuzzy();
       void runFuzzyTimes(int);
       void evaluateFitness();
       void runEpoch(Population&);
       Population* runOnceOn(Population&);
       void runEpoch2(Population&);
       void setTestType(int);

//       ~GeneticAlgorithm();	

       GeneticAlgorithm(FitnessFunction *_Func, CrossOverOperation *Cross, 
			MutationOperator *Mutator, ParentSelector *PSelObject,
                        Recombinator *RecomObj, Initializer *InitObject,
			Sampler *_MySampler = NULL,
			GAOperationsBag  *CrossOps=NULL,
			GAOperationsBag  *MutOps=NULL);

       void SetMaxGeneration(int _m);

       void SetPopulationSize(int _size);

       void SetMutationProb(float _ProbMut);

       void SetCrossProb(float _ProbCross);

   private:
       FitnessFunction *Function;
       CrossOverOperation *CrossOver;
       MutationOperator *Mutation;
       GAOperationsBag  *CrossOverOprs;
       GAOperationsBag  *MutationOprs;
       int MaxGeneration;
       int PopulationSize;
       float MutationProbability;
       float CrossoverProbability;
       Population MyPopulation;
       ParentSelector  *ParentSelectorObject;
       Recombinator    *RecombinatorObject;
       Initializer     *InitializerObject;
       Sampler         *MySampler;
       RandomGenerator Random;
       int  testNum;
       int  testType;

       void writeStatistics(int,char*);
       void updateOperatorProbabilities(GAOperationsBag&, const Population&,
       OP_TYPE);
};
#endif
